/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */

#include "OpenTranslation.h"
#include "DocumentSystem.h"


/*
 *
 */
bool OpenTranslation::execute(const Glib::ustring &filename, const Glib::ustring &encoding)
{
	try
	{
		Document *current = DocumentSystem::getInstance().getCurrentDocument();

		if(current == NULL)
		{
			std::cerr << "Document == NULL" << std::endl;
			return false;
		}

		Document *doc = new Document;

		doc->setCharset(encoding);
		if(doc->open(filename))
		{
			Gtk::TreeIter it1 = current->get_subtitle_model()->getFirst();
			Gtk::TreeIter it2 = doc->get_subtitle_model()->getFirst();

			while(it1 && it2)
			{
				SubtitleModifier sub1(it1);
				SubtitleModifier sub2(it2);

				sub1.set_translation(sub2.get_text());

				++it1;
				++it2;
			}
			
			delete doc;

			return true;
		}
		
		delete doc;
		return false;
	}
	catch(...)
	{
	}
	return false;
}
