#include "Player.h"
#include <iostream>
#include "SubtitleTime.h"
#include <gdk/gdkx.h>
#include "Config.h"

/*
 *
 */
Player::Player()
{
	m_widget_socket = NULL;
	m_widget_pid = 0;
	m_state = DISABLE;
}

/*
 *
 */
Player::~Player()
{
	m_connection_timeout.disconnect();
}

/*
 *
 */
bool Player::is_valid()
{
	return m_state != DISABLE;
}

/*
 *
 */
void Player::set_widget(Gtk::Widget *socket)
{
	g_return_if_fail(socket);

	m_widget_socket = socket;
	//m_widget_pid = GDK_WINDOW_XID(socket->get_window()->gobj());
}

/*
 *
 */
Gtk::Widget* Player::get_widget()
{
	//g_return_val_if_fail(m_widget_socket, NULL);
	if(m_widget_socket == NULL)
		return NULL;

	return m_widget_socket;
}

/*
 *
 */
Glib::Pid Player::get_widget_pid()
{
	//g_return_val_if_fail(m_widget_pid, 0);
	if(m_widget_socket == NULL)
		return 0;

	Glib::RefPtr<Gdk::Window> window = m_widget_socket->get_window();

	if(window)
		m_widget_pid = GDK_WINDOW_XID(window->gobj());

	return m_widget_pid;
}

/*
 *
 */
void Player::set_state(Player::State state)
{
	std::cout << "set_state:" << state << std::endl;

	if(!m_connection_timeout)
	{
		// read config for timeout value
		int msec;
		Config::getInstance().get_value_int("video-player", "timeout", msec, 100);

		// 
		m_connection_timeout = Glib::signal_timeout().connect(
				sigc::mem_fun(*this, &Player::on_timeout), msec);
		m_connection_timeout.block();
	}

	m_state = state;

	if(state == DISABLE )
		m_connection_timeout.block();
	else if(state == PAUSED)
		m_connection_timeout.block();
	else if(state == PLAYING)
	{
		m_connection_timeout.unblock();
		m_signal_timeout();
	}
	m_signal_state_changed(state);

}

Player::State Player::get_state()
{
	return m_state;
}

/*
 *	signaux
 */
sigc::signal<void>& Player::get_signal_timeout()
{
	return m_signal_timeout;
}

/*
 *
 */
sigc::signal<void, Player::State>& Player::get_signal_state_changed()
{
	return m_signal_state_changed;
}

/*
 *
 */
bool Player::on_timeout()
{
	m_signal_timeout();
	return is_playing();
}
