#include <gtkmm.h>
#include <libglademm/xml.h>
#include "utility.h"
#include "DocumentSystem.h"
#include "ScaleUI.h"
#include "actions/Scale.h"

/*
 *
 */
class DialogScale : public Gtk::Dialog
{
public:
	DialogScale(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::Dialog(cobject)
	{
		refGlade->get_widget("spin-first-number", m_spinFirstNumber);
		refGlade->get_widget_derived("spin-first-current-time", m_spinFirstCurrentTime);
		refGlade->get_widget_derived("spin-first-new-time", m_spinFirstNewTime);

		refGlade->get_widget("spin-last-number", m_spinLastNumber);
		refGlade->get_widget_derived("spin-last-current-time", m_spinLastCurrentTime);
		refGlade->get_widget_derived("spin-last-new-time", m_spinLastNewTime);

		// signaux
		m_spinFirstNumber->signal_value_changed().connect(
				sigc::mem_fun(*this, &DialogScale::on_spin_first_number_changed));
		m_spinLastNumber->signal_value_changed().connect(
				sigc::mem_fun(*this, &DialogScale::on_spin_last_number_changed));

	}

	void execute()
	{
		if(init_with_document() == false)
		{
			return;
		}

		show();

		if(run() == Gtk::RESPONSE_OK)
		{
			Document *doc = DocumentSystem::getInstance().getCurrentDocument();

			SubtitleModelPtr model = doc->get_subtitle_model();
			/*
			unsigned int firstNumber = (unsigned int)m_spinFirstNumber->get_value();
			unsigned int lastNumber = (unsigned int)m_spinLastNumber->get_value();


			SubtitleModifier firstSubtitle(model->find(firstNumber));
			SubtitleModifier lastSubtitle(model->find(lastNumber));

			//if(firstSubtitle && lastSubtitle)
			*/
			SubtitleTime src1( (long int)m_spinFirstCurrentTime->get_value() );
			SubtitleTime dest1( (long int)m_spinFirstNewTime->get_value() );
			
			SubtitleTime src2( (long int)m_spinLastCurrentTime->get_value() );
			SubtitleTime dest2( (long int)m_spinLastNewTime->get_value() );

			// apply change
			Scale scale;
			scale.execute(*doc, src1, dest1, src2, dest2);
		}

		hide();
	}
protected:

	bool init_with_document()
	{
		Document *doc = DocumentSystem::getInstance().getCurrentDocument();

		g_return_val_if_fail(doc, false);

		unsigned int subtitle_size = doc->get_subtitle_model()->getSize();

		if(subtitle_size == 0)
		{
			dialog_warning(
					_("You can't use <i>scale</i> with this document."), 
					build_message("The document <b>%s</b> has not subtitle, it's empty.", doc->getName().c_str()));
			return false;
		}
		m_spinFirstNumber->set_range(1, subtitle_size);
		m_spinLastNumber->set_range(1, subtitle_size);

		m_spinFirstNumber->set_value(1);
		m_spinLastNumber->set_value(subtitle_size);

		return true;
	}

	/*
	 *
	 */
	void on_spin_first_number_changed()
	{
		Document *doc = DocumentSystem::getInstance().getCurrentDocument();

		if(doc)
		{
			unsigned int i = (unsigned int)m_spinFirstNumber->get_value();

			SubtitleModifier sub(doc->get_subtitle_model()->find(i));
			
			if(sub)
				init_spin(sub, m_spinFirstCurrentTime, m_spinFirstNewTime);
		}
	}

	/*
	 *
	 */
	void on_spin_last_number_changed()
	{
		Document *doc = DocumentSystem::getInstance().getCurrentDocument();

		if(doc)
		{
			unsigned int i = (unsigned int)m_spinLastNumber->get_value();

			SubtitleModifier sub(doc->get_subtitle_model()->find(i));
			
			if(sub)
				init_spin(sub, m_spinLastCurrentTime, m_spinLastNewTime);
		}
	}
	
	/*
	 *
	 */
	void init_spin(const SubtitleModifier &subtitle, SpinButtonTiming *current, SpinButtonTiming *newtime)
	{
		long int time = subtitle.get_start().totalmsecs;

		current->set_value(time);
		current->set_range(time, time);
		newtime->set_value(time);
	}

protected:
	Gtk::SpinButton*	m_spinFirstNumber;
	SpinButtonTiming*	m_spinFirstCurrentTime;
	SpinButtonTiming*	m_spinFirstNewTime;

	Gtk::SpinButton*	m_spinLastNumber;
	SpinButtonTiming*	m_spinLastCurrentTime;
	SpinButtonTiming*	m_spinLastNewTime;
};


/*
 *
 */
ScaleUI::ScaleUI()
{
}

/*
 *
 */
ScaleUI::~ScaleUI()
{
}

/*
 *
 */
void ScaleUI::execute()
{
	DialogScale *dialog = NULL;

	Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(get_share_dir("dialog-scale.glade"));

	refXml->get_widget_derived("dialog-scale", dialog);

	dialog->execute();

	delete dialog;
}
