#include "SplitDocumentUI.h"
#include <gtkmm.h>
#include "utility.h"
#include "DocumentSystem.h"

/*
 *
 */
class DialogSplitDocument : public Gtk::Dialog
{
public:

	/*
	 *
	 */
	DialogSplitDocument(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::Dialog(cobject)
	{
		refGlade->get_widget("spin-number", m_spinNumber);

		set_default_response(Gtk::RESPONSE_OK);
	}

	/*
	 *
	 */
	void execute()
	{
		Document *doc = DocumentSystem::getInstance().getCurrentDocument();

		g_return_if_fail(doc);

		unsigned int size = doc->get_subtitle_model()->getSize();

		if(size == 0)
		{
			dialog_warning(
					_("You can't use <i>split</i> with this document."), 
					build_message("The document <b>%s</b> has not subtitle, it's empty.", doc->getName().c_str()));
			return;
		}

		m_spinNumber->set_range(1,size);

		show();

		if(run() == Gtk::RESPONSE_OK)
		{
			unsigned int number = (unsigned int)m_spinNumber->get_value();

			split_doc(doc, number);
		}
		
		hide();
	}
protected:

	/*
	 *	coupe le doc en 2 et retourne le nouveau
	 */
	Document *split_doc(Document *doc, unsigned int number)
	{
		// creation du nouveau document par copy
		Document *newdoc = new Document(*doc);
		// même nom plus -part2
		newdoc->setFilename(newdoc->getFilename() + "-par2");
		// on supprime les sous-titres précédant
		newdoc->get_subtitle_model()->remove(1, number-1);
		newdoc->get_subtitle_model()->rebuild_column_num();

		DocumentSystem::getInstance().append(newdoc);

		// on supprime ensuite les sous-titres utiliser par le nouveau document
		doc->get_subtitle_model()->remove(number, doc->get_subtitle_model()->getSize());

		return newdoc;
	}

	/*
	 *
	 */
	void adjust_time(Document *doc, const SubtitleTime &time)
	{
	}
protected:
	Gtk::SpinButton* m_spinNumber;
};





/*
 *
 */
void SplitDocumentUI::execute()
{
	DialogSplitDocument *dialog = NULL;

	Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(get_share_dir("dialog-split-document.glade"));

	refXml->get_widget_derived("dialog-split-document", dialog);

	dialog->execute();

	delete dialog;
}
