#include "WaveformSystem.h"
#include <cairomm/context.h>
#include <gst/gst.h>
#include "utility.h"
#include "DocumentSystem.h"

#include "ActionSystem.h"
#include "gui/OpenWaveformUI.h"
#include "gui/WaveformGeneratorUI.h"

#include <gtk/gtk.h>


/*
 *	use directly gtk beacause gtkmm doesn't work!
 */
bool on_gtk_scroll_event(GtkWidget *w, GdkEventScroll *ev, gpointer data)
{
	if(ev->direction == GDK_SCROLL_UP)
		gtk_range_set_value(GTK_RANGE(w), gtk_range_get_value(GTK_RANGE(w)) + 1);
	else if(ev->direction == GDK_SCROLL_DOWN)
		gtk_range_set_value(GTK_RANGE(w), gtk_range_get_value(GTK_RANGE(w)) - 1);
	else
		return false;

	return true;
}

/*
 *
 */
WaveformSystem::WaveformSystem(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
:Gtk::Frame(cobject)
{
	m_zoom = m_scale = 0;
	m_display_time_info = false;

	se_debug(SE_DEBUG_WAVEFORM);

	m_scrolling_with_cursor = true;
	m_scrolling_with_selection = true;
	
	m_currentDocument = NULL;
	m_playerVideo = NULL;

	refGlade->get_widget("drawing-waveform", m_drawingWaveform);
	refGlade->get_widget("hscrollbar-waveform", m_hscrollbarWaveform);

	refGlade->get_widget("vscale-zoom", m_sliderZoom);
	refGlade->get_widget("vscale-scale", m_sliderScale);

	// waveform
	m_drawingWaveform->add_events(
			Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON_MOTION_MASK | Gdk::SCROLL_MASK);

	m_drawingWaveform->signal_expose_event().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_expose_event_waveform));
	
	m_drawingWaveform->signal_configure_event().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_configure_event_waveform));

	m_drawingWaveform->signal_button_press_event().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_button_press_event_waveform));
	
	m_drawingWaveform->signal_button_release_event().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_button_release_event_waveform));

	m_drawingWaveform->signal_motion_notify_event().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_motion_notify_event_waveform));

	m_drawingWaveform->signal_scroll_event().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_scroll_event_waveform));

	// scrollbar
	m_hscrollbarWaveform->signal_value_changed().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_hscrollbar_waveform_value_changed));

	// slider
	m_sliderZoom->signal_value_changed().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_slider_zoom_changed));
	m_sliderZoom->add_events(Gdk::SCROLL_MASK);
	//m_sliderZoom->signal_scroll_event().connect(
	//		sigc::bind(sigc::mem_fun(*this, &WaveformSystem::on_slider_scroll_event), m_sliderZoom));
	g_signal_connect(G_OBJECT(m_sliderZoom->gobj()), "scroll-event", G_CALLBACK(on_gtk_scroll_event), NULL);


	m_sliderScale->signal_value_changed().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_slider_scale_changed));
	m_sliderScale->add_events(Gdk::SCROLL_MASK);
	//m_sliderScale->signal_scroll_event().connect(
	//		sigc::bind(sigc::mem_fun(*this, &WaveformSystem::on_slider_scroll_event), m_sliderScale));
	g_signal_connect(G_OBJECT(m_sliderScale->gobj()), "scroll-event", G_CALLBACK(on_gtk_scroll_event), NULL);


	//
	DocumentSystem::getInstance().signal_current_document_changed().connect(
			sigc::mem_fun(*this, &WaveformSystem::init_with_document));

	ActionSystem::getInstance().signal_emit().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_action_emit));
	
	//
	set_sensitive(false);

	int zoom, scale;
	Config::getInstance().get_value_int("waveform", "zoom", zoom, 3);
	Config::getInstance().get_value_int("waveform", "scale", scale, 2);

	set_zoom(zoom);
	set_scale(scale);

	// config
	// load_config();

	Config::getInstance().signal_changed("waveform").connect(
			sigc::mem_fun(*this, &WaveformSystem::on_config_waveform_changed));

	// on n'utilise pas width puisque le waveform 
	// doit prendre toute la place disponible
	//int height;
	//if(Config::getInstance().get_value_int("waveform", "height", height))
	//{
	//	set_size_request(-1, height);
	//}
}

/*
 *
 */
WaveformSystem::~WaveformSystem()
{
	se_debug(SE_DEBUG_WAVEFORM);
//	m_waveform.clear();
}

/*
 *
 */
void WaveformSystem::set_video_player(Player *player)
{
	se_debug(SE_DEBUG_WAVEFORM);

	m_playerVideo = player;

	m_playerVideo->get_signal_timeout().connect(
			sigc::mem_fun(*this, &WaveformSystem::on_video_timeout));
}

/*
 *
 */
void WaveformSystem::load_config()
{
	se_debug(SE_DEBUG_WAVEFORM);

	m_color_background[0]=0.9;
	m_color_background[1]=0.9;
	m_color_background[2]=0.9;
	m_color_background[3]=1.0;
	
	m_color_play_line[0]=0.0;
	m_color_play_line[1]=0.0;
	m_color_play_line[2]=0.0;
	m_color_play_line[3]=1.0;

	m_color_wave[0]=0.0;
	m_color_wave[1]=0.7;
	m_color_wave[2]=0.0;
	m_color_wave[3]=1.0;

	m_color_wave_fill[0]=0.0;
	m_color_wave_fill[1]=0.5;
	m_color_wave_fill[2]=0.0;
	m_color_wave_fill[3]=1.0;

	m_color_marker[0]=0.4;
	m_color_marker[1]=0.4;
	m_color_marker[2]=0.4;
	m_color_marker[3]=0.5;

	m_color_marker_hightlight[0]=0.3;
	m_color_marker_hightlight[1]=0.5;
	m_color_marker_hightlight[2]=0.8;
	m_color_marker_hightlight[3]=0.5;

	m_display_waveform_fill = true;
	m_display_background = false;
	m_scrolling_with_cursor = true;
	m_scrolling_with_selection = true;

	Config &cfg = Config::getInstance();

#define read(value, name, default_value) \
	{ \
		Glib::ustring string; \
		cfg.get_value_string("waveform", name, string, default_value); \
			sscanf(string.c_str(), "%f;%f;%f;%f", &value[0], &value[1], &value[2], &value[3]);\
	}

	read(m_color_background, "color-background", build_message("%f;%f;%f;%f", 1.0, 1.0, 1.0, 1.0));
	read(m_color_play_line, "color-play-line", build_message("%f;%f;%f;%f", 0.0, 0.0, 0.0, 1.0));
	read(m_color_wave, "color-wave", build_message("%f;%f;%f;%f", .0, 0.7, 0.0, 1.0));
	read(m_color_wave_fill, "color-wave-fill", build_message("%f;%f;%f;%f", 0.0, 0.5, 0.0, 1.0));
	read(m_color_marker, "color-marker", build_message("%f;%f;%f;%f", .4, .4, .4, .5));
	read(m_color_marker_hightlight, "color-marker-hightlight", build_message("%f;%f;%f;%f", 0.3, 0.5, 0.8, 0.5));
	
	cfg.get_value_bool("waveform", "display-waveform-fill", m_display_waveform_fill, true);
	cfg.get_value_bool("waveform", "display-background", m_display_background, false);
	cfg.get_value_bool("waveform", "scrolling-with-cursor", m_scrolling_with_cursor, true);
	cfg.get_value_bool("waveform", "scrolling-with-selection", m_scrolling_with_selection, true);
}

/*
 *
 */
bool WaveformSystem::open_media(const Glib::ustring &uri)
{
	se_debug_message(SE_DEBUG_WAVEFORM, "uri=%s", uri.c_str());

	if(m_waveform)
		m_waveform.clear();

	m_waveform = Glib::RefPtr<Waveform>(new Waveform);
	
	bool state = m_waveform->open(uri);

#warning "FIXME: show/hide Config/MenuBar"
	if(state)
	{
		set_sensitive(true);
		init_scrollbarWaveform();
		
		Config::getInstance().set_value_bool("interface", "display-waveform", true);
		
		show();
	}
	else
	{
		m_waveform.clear();

		set_sensitive(false);
		hide();
	}

	redraw_all();

	return state;
}

/*
 *
 */
void WaveformSystem::set_zoom(int zoom)
{
	se_debug_message(SE_DEBUG_WAVEFORM, "zoom=%d", zoom);

	utility::clamp(zoom, 1, 1000);

	if(m_zoom == zoom)
		return;

	m_zoom = zoom;

	Config::getInstance().set_value_int("waveform", "zoom", m_zoom);

	m_sliderZoom->set_value(zoom);

	init_scrollbarWaveform();

	redraw_all();
}

/*
 *
 */
void WaveformSystem::set_scale(int scale)
{
	se_debug_message(SE_DEBUG_WAVEFORM, "scale=%d", scale);

	utility::clamp(scale, 1, 10);

	if(m_scale == scale)
		return;

	m_scale = scale;

	m_sliderScale->set_value(scale);

	Config::getInstance().set_value_int("waveform", "scale", m_scale);

	redraw_waveform();
}

/*
 *
 */
void WaveformSystem::on_hscrollbar_waveform_value_changed()
{
	se_debug(SE_DEBUG_WAVEFORM);

	redraw_all();
}

/*
 *
 */
bool WaveformSystem::on_configure_event_waveform(GdkEventConfigure *ev)
{
	se_debug(SE_DEBUG_WAVEFORM);

	init_scrollbarWaveform();
	
	redraw_all();

	// on utilise pas width puisque le waveform prend toute la place dispo
	//Config::getInstance().set_value_int("waveform", "width", get_width());
	//Config::getInstance().set_value_int("waveform", "height", get_height());

	return false;
}

/*
 *	retourne le temps en msec (SubtitleTime.totalmsecs) 
 *	par rapport à la position dans la drawingarea
 */
long WaveformSystem::__get_time_by_pos(int pos)
{
	float width = (float)m_drawingWaveform->get_width() * get_zoom();
	float percent = ((float)pos / width);
	float gsttime = ((float)m_waveform->get_duration() * percent);
	return SubtitleTime(long(gsttime / GST_MSECOND)).totalmsecs;
}

/*
 *	return la position dans la drawingarea par rapport
 *	au temps en msec (SubtitleTime.totalmsecs)
 */
int WaveformSystem::__get_pos_by_time(long time)
{
	gint64 gsttime = time * GST_MSECOND;
	float percent = ((float)gsttime / (float)m_waveform->get_duration());
	float width = (float)m_drawingWaveform->get_width() * get_zoom();
	float pos = width * percent;
	pos = CLAMP(pos, 0, width);
	return (int)pos;
}

/*
 *
 */
void WaveformSystem::__draw_timeline(Cairo::RefPtr<Cairo::Context> &cr, const Gdk::Rectangle &rect)
{
	int width = rect.get_width();
	int height = rect.get_height();

	// background
	cr->set_source_rgb(1,1,1);
	cr->rectangle(0, 0, width, height);
	cr->fill();

	// line H
	cr->set_source_rgb(0,0,0);
	cr->move_to(0, height);
	cr->line_to(width, height);
	cr->stroke();

	long sec_1 = SubtitleTime(0,0,1,0).totalmsecs;
	long sec_5 = SubtitleTime(0,0,5,0).totalmsecs;
	long sec_10 = SubtitleTime(0,0,10,0).totalmsecs;

	while( __get_pos_by_time(sec_5) < 60)
	{
		sec_1  *= 2;
		sec_5  *= 2;
		sec_10 *= 2;
	}
	__draw_timeline_msecs(cr, sec_1, rect, height - 3);
	__draw_timeline_msecs(cr, sec_5, rect, height - 5);
	__draw_timeline_msecs(cr, sec_10, rect, height - 8);

	__draw_timeline_time(cr, sec_5, rect);

}

void WaveformSystem::__draw_timeline_msecs(Cairo::RefPtr<Cairo::Context> &cr, long msec, const Gdk::Rectangle &rect, int upper)
{
	int height = rect.get_height();

	int start_area = get_start_area();
	int end_area = get_end_area();

	long start = __get_time_by_pos(start_area);
	long end = __get_time_by_pos(end_area);

	long diff = start % msec;

	start -= diff;

	// TODO: FIXME
	if(__get_pos_by_time(start) < start_area)
		start += msec;

	for(long t = start; t< end; t+=msec)
	{
		int x = __get_pos_by_time(t) - start_area;

		cr->move_to(x, height);
		cr->line_to(x, upper);
	}
	cr->stroke();
}

/*
 *
 */
void WaveformSystem::__draw_timeline_time(Cairo::RefPtr<Cairo::Context> &cr, long msec, const Gdk::Rectangle &rect)
{
	if(msec > 0)
	{
		cr->set_source_rgb(0,0,0);
		cr->set_font_size(13);

		int height = rect.get_height() - 15;

		int start_area = get_start_area();
		int end_area = get_end_area();

		long start = __get_time_by_pos(start_area);
		long end = __get_time_by_pos(end_area);

		long diff = start % msec;

		start -= diff;

		Cairo::TextExtents ex;
		cr->get_text_extents(Gst::time_to_string(0), ex);

		//int sub_width = start_area - ex.width * 0.5;

		for(long t = start; t< end; t+=msec)
		{
			int x = __get_pos_by_time(t) - start_area;

			cr->move_to(x - ex.width * 0.5, height);
	
			cr->show_text(SubtitleTime(t).str().substr(0,7));
		}
	}
}

/*
 *	Waveform
 */
bool WaveformSystem::on_expose_event_waveform(GdkEventExpose *ev)
{
	se_debug(SE_DEBUG_WAVEFORM);

	Glib::RefPtr<Gdk::Window> window = m_drawingWaveform->get_window();
	
	if(!window)
		return false;

	if(!m_waveform)
		return false;

	static bool loading = false;

	if(!loading)
	{
		loading = true;
		load_config();
	}


	// widget size
	Gtk::Allocation allocation = m_drawingWaveform->get_allocation();
    
	int width = allocation.get_width();
  int height = allocation.get_height();

	Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();


	// waveform
	if(!m_waveform_surface)
	{
		//
		m_waveform_surface = Cairo::Surface::create(cr->get_target(), Cairo::CONTENT_COLOR_ALPHA, width, height);
		Cairo::RefPtr<Cairo::Context> context = Cairo::Context::create(m_waveform_surface);

		if(m_display_background)
		{
			context->set_source_rgba(
					m_color_background[0], m_color_background[1], m_color_background[2], m_color_background[3]);

			context->paint();
		}

		__draw_timeline(context, Gdk::Rectangle(0,0, width, 30));

		height = height - 30;
	
		unsigned int n_channels = m_waveform->get_n_channels();

		for(unsigned int i=0; i<n_channels; ++i)
		{
			draw_channels(context, i, i*(height/n_channels) + 30, (height/n_channels));
		}
	}

	// marker
	if(!m_waveform_marker_surface)
	{
		m_waveform_marker_surface = Cairo::Surface::create(cr->get_target(), Cairo::CONTENT_COLOR_ALPHA, width, height);
		Cairo::RefPtr<Cairo::Context> context = Cairo::Context::create(m_waveform_marker_surface);

		if(m_currentDocument)// = DocumentSystem::getInstance().getCurrentDocument())
		{
			Gdk::Rectangle rec(0,0, width, height);
			draw_markers(context, rec, m_currentDocument);
		}
	}

	if(m_waveform_surface)
	{
		cr->set_source(m_waveform_surface, 0,0);
		cr->paint();
	}

	if(m_waveform_marker_surface)
	{
		cr->set_source(m_waveform_marker_surface, 0, 30);
		cr->paint();
	}

	if(m_playerVideo && m_playerVideo->is_valid())
	{
		// current play
		gint64 gst_position = m_playerVideo->get_position() * GST_MSECOND;
		int position = get_pos_by_time(gst_position) - get_start_area();

		cr->set_source_rgba(m_color_play_line[0], m_color_play_line[1], m_color_play_line[2], m_color_play_line[3]);
		cr->move_to(position, 30);
		cr->line_to(position, height+30);
		cr->stroke();
	}

	// display time
	if(m_currentDocument && m_display_time_info)
	{
		gint x=0, y=0;
		Gdk::ModifierType mod;
		Glib::RefPtr<Gdk::Window> win = window->get_pointer(x,y, mod);
		
		//if(	x > ev->area.x && x < ev->area.width &&
		//		y > ev->area.y && y < ev->area.height )
		{
			//if((mod & Gdk::BUTTON1_MASK) || (mod & Gdk::BUTTON3_MASK))
			{
				// current time is displayed 
				guint pos = (guint)x + get_start_area();
				gint64 gst_time = get_time_by_pos(pos);

				SubtitleTime time ( gst_time / GST_MSECOND);

				cr->set_font_size(12);
				cr->set_source_rgb(0,0,0);

				Cairo::TextExtents ex;
				cr->get_text_extents(SubtitleTime::null(), ex);

				cr->move_to(x - ex.width * 0.5, y - 5);
				cr->show_text(time.str());

				// duration is displayed in center of subtitle.
				SubtitleModifier sub(m_currentDocument->get_subtitle_view()->getSelected());
				if(sub)
				{
					int center = __get_pos_by_time((sub.get_start() + (sub.get_end() - sub.get_start()) * 0.5).totalmsecs) - get_start_area();

					cr->move_to(center - ex.width * 0.5, y + 30);
					cr->show_text(sub.get_duration().str());
				}
			}
		}
	}
	return true;
}

/*
 *
 */
bool WaveformSystem::on_button_press_event_waveform(GdkEventButton *ev)
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(!m_waveform)
		return true;

	// on recupere la position et on transforme en temps dans la waveform

	// on prend en compte la hscrollbar
	guint position = (guint)ev->x + get_start_area();
	
	gint64 gst_time = get_time_by_pos(position);

	// position to time
	SubtitleTime time( gst_time / GST_MSECOND);

	if(ev->button == 2  && !(ev->state & Gdk::CONTROL_MASK) && m_playerVideo)
	{
		if(m_playerVideo->is_valid())
		{
			m_playerVideo->seek(time.totalmsecs);
			if(m_playerVideo->is_playing() == false)
				m_playerVideo->play();
		}
		return true;
	}
	else if(ev->button == 2 && ev->state & Gdk::CONTROL_MASK && m_currentDocument)
	{
		SubtitleModifier sub(m_currentDocument->get_subtitle_model()->find(time));
		if(sub)
		{
			m_currentDocument->get_subtitle_view()->select_and_set_cursor(sub.get_iter());
		}
		return true;
	}

	if(m_currentDocument == NULL)
		return true;

	SubtitleModifier subtitle(m_currentDocument->get_subtitle_view()->getSelected());
	
	if(!subtitle)
		return true;

	if(ev->button == 1 && ev->state & Gdk::CONTROL_MASK)
	{
		// on décale la début et la fin du précédent sous-titre

		// on récupere l'espace entre les deux sous-titres
		SubtitleTime diff = time - subtitle.get_start();

		subtitle.set_start(time);

		SubtitleModifier previous(m_currentDocument->get_subtitle_model()->find_previous(subtitle.get_iter()));
		if(previous)
		{
			previous.set_end(previous.get_end() + diff);
		}
		m_currentDocument->emit_signal("subtitle-time-changed");
	}
	else if(ev->button == 1)
	{
		subtitle.set_start(time);
		m_currentDocument->emit_signal("subtitle-time-changed");
	}
	else if(ev->button == 3 && ev->state & Gdk::CONTROL_MASK)
	{
		// on décale la fin et le début du prochain sous-titre

		// on récupere l'espace entre les deux sous-titres
		SubtitleTime diff = time - subtitle.get_end();

		subtitle.set_end(time);

		SubtitleModifier next(m_currentDocument->get_subtitle_model()->find_next(subtitle.get_iter()));
		if(next)
		{
			next.set_start(next.get_start() + diff);
		}
		m_currentDocument->emit_signal("subtitle-time-changed");
	}
	else if(ev->button == 3)
	{
		subtitle.set_end(time);
		m_currentDocument->emit_signal("subtitle-time-changed");
	}

	m_display_time_info = true;

	return true;
}

/*
 *
 */
bool WaveformSystem::on_button_release_event_waveform(GdkEventButton *ev)
{
	m_display_time_info = false;

	m_drawingWaveform->queue_draw();
	return true;
}

/*
 *
 */
bool WaveformSystem::on_motion_notify_event_waveform(GdkEventMotion *ev)
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(m_currentDocument == NULL || !m_waveform)
		return true;

	SubtitleModifier subtitle(m_currentDocument->get_subtitle_view()->getSelected());
	
	if(!subtitle)
		return true;

	// on recupere la position et on transforme en temps dans la waveform

	// on prend en compte la hscrollbar
	guint position = CLAMP((guint)ev->x, 0, (guint)m_drawingWaveform->get_width()) + get_start_area();

	// position to time
	SubtitleTime time( get_time_by_pos(position) / GST_MSECOND);

	if(ev->state & Gdk::BUTTON1_MASK && ev->state & Gdk::CONTROL_MASK)
	{
		// on décale la début et la fin du précédent sous-titre

		// on récupere l'espace entre les deux sous-titres
		SubtitleTime diff = time - subtitle.get_start();

		subtitle.set_start(time);

		SubtitleModifier previous(m_currentDocument->get_subtitle_model()->find_previous(subtitle.get_iter()));
		if(previous)
		{
			previous.set_end(previous.get_end() + diff);
		}
		m_currentDocument->emit_signal("subtitle-time-changed");
	}
	else if(ev->state & Gdk::BUTTON1_MASK && ev->state & Gdk::SHIFT_MASK)
	{
		subtitle.set_end(time);
		m_currentDocument->emit_signal("subtitle-time-changed");
	}
	else if(ev->state & Gdk::BUTTON1_MASK)
	{
		subtitle.set_start(time);
		m_currentDocument->emit_signal("subtitle-time-changed");
	}
	else if(ev->state & Gdk::BUTTON3_MASK && ev->state & Gdk::CONTROL_MASK)
	{
		// on décale la fin et le début du prochain sous-titre

		// on récupere l'espace entre les deux sous-titres
		SubtitleTime diff = time - subtitle.get_end();

		subtitle.set_end(time);

		SubtitleModifier next(m_currentDocument->get_subtitle_model()->find_next(subtitle.get_iter()));
		if(next)
		{
			next.set_start(next.get_start() + diff);
		}
		m_currentDocument->emit_signal("subtitle-time-changed");
	}
	else if(ev->state & Gdk::BUTTON3_MASK)
	{
		subtitle.set_end(time);
		m_currentDocument->emit_signal("subtitle-time-changed");
	}
	return true;
}

/*
 *
 */
bool WaveformSystem::on_scroll_event_waveform(GdkEventScroll *ev)
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(!m_waveform)
		return true;

	// scale
	if(ev->direction == GDK_SCROLL_UP && ev->state & Gdk::SHIFT_MASK)
	{
		set_scale(m_scale + 1);
	}
	else if(ev->direction == GDK_SCROLL_DOWN && ev->state & Gdk::SHIFT_MASK)
	{
		set_scale(m_scale - 1);
	}
	// zoom
	else if(ev->direction == GDK_SCROLL_UP && ev->state & Gdk::CONTROL_MASK)
	{
		int center = get_start_area() + (int)((get_end_area() - get_start_area()) * 0.5);
		long time = __get_time_by_pos(center);

		zoom_in();
		
		scroll_to_position_and_center(__get_pos_by_time(time));
	}
	else if(ev->direction == GDK_SCROLL_DOWN && ev->state & Gdk::CONTROL_MASK)
	{
		int center = get_start_area() + (int)((get_end_area() - get_start_area()) * 0.5);
		long time = __get_time_by_pos(center);

		zoom_out();
		
		scroll_to_position_and_center(__get_pos_by_time(time));
	}
	// scroll
	else if(ev->direction == GDK_SCROLL_UP)
	{
		double page_size = m_hscrollbarWaveform->get_adjustment()->get_page_size();
		double delta = pow(page_size, 2.0 / 3.0);
		m_hscrollbarWaveform->set_value(m_hscrollbarWaveform->get_value() - delta);
	}
	else if(ev->direction == GDK_SCROLL_DOWN)
	{
		double page_size = m_hscrollbarWaveform->get_adjustment()->get_page_size();
		double delta = pow(page_size, 2.0 / 3.0);
		m_hscrollbarWaveform->set_value(m_hscrollbarWaveform->get_value() + delta);
	}
	
	return true;
}

/*
 *   Glib::signal_timeout().connect( sigc::mem_fun(*this, &WaveformSystem::onSecondElapsed), 1000);
 */
void WaveformSystem::redraw_waveform()
{
	se_debug(SE_DEBUG_WAVEFORM);

	m_waveform_surface.clear();

	m_drawingWaveform->queue_draw();
}

/*
 *	efface simplement la surface (cairo)
 *	puis on demande au widget (drawing area)
 *	de rafraichir l'affichage
 *	pour dessiner la surface
 */
void WaveformSystem::redraw_marker()
{
	se_debug(SE_DEBUG_WAVEFORM);

	m_waveform_marker_surface.clear();

	m_drawingWaveform->queue_draw();
}

/*
 *	efface simplement les surfaces (cairo)
 *	puis on demande au widget (drawing area)
 *	de rafraichir l'affichage
 *	pour dessiner la surface
 */
void WaveformSystem::redraw_all()
{
	se_debug(SE_DEBUG_WAVEFORM);

	m_waveform_surface.clear();
	m_waveform_marker_surface.clear();

	m_drawingWaveform->queue_draw();
}


/*
 *
 */
int WaveformSystem::get_zoom()
{
	//se_debug_message(SE_DEBUG_WAVEFORM, "zoom=%f", m_zoom);

	return m_zoom;
}

/*
 *	passe d'une position dans l'area à une position dans le channel
 *	channel != len
 */
int WaveformSystem::area_to_waveform_channel(int pos)
{
	//se_debug(SE_DEBUG_WAVEFORM);

	// taille total du widget
	float width = (float)m_drawingWaveform->get_width() * get_zoom();

	//on bascule en % a partir de l'espace area
	float percent = (float)pos / width;

	// du % on rebascule sur l'espace channel (waveform)
	return (int)((float)m_waveform->get_size() * percent);
}

/*
 *
 */
double WaveformSystem::area_to_waveform_channel_2(int pos)
{
	//se_debug(SE_DEBUG_WAVEFORM);

	// taille total du widget
	double width = (double)m_drawingWaveform->get_width() * get_zoom();

	//on bascule en % a partir de l'espace area
	double percent = (double)pos / width;

	// du % on rebascule sur l'espace channel (waveform)
	return (double)m_waveform->get_size() * percent;
}

/*
 *	init la scrollbar selon ça taille et le zoom
 */
void WaveformSystem::init_scrollbarWaveform()
{
	se_debug(SE_DEBUG_WAVEFORM);

	double upper = m_hscrollbarWaveform->get_adjustment()->get_upper();
	double old_value = m_hscrollbarWaveform->get_value();

	guint width = m_drawingWaveform->get_width();

	Gtk::Adjustment *adj = m_hscrollbarWaveform->get_adjustment();

	adj->set_page_size((double)width);
	adj->set_page_increment(width);
	adj->set_step_increment(width);

	m_hscrollbarWaveform->set_range(0, width * get_zoom());

	if(upper > 0)
		m_hscrollbarWaveform->set_value(width * get_zoom() * (old_value / upper));
}

/*
 *	retourne la position du debut de l'affichage
 *	(prise en compte du decalage avec la scrollbar)
 */
int WaveformSystem::get_start_area()
{
	//se_debug(SE_DEBUG_WAVEFORM);

	return (int)m_hscrollbarWaveform->get_value();
}

/*
 *	retourne la position de la fin de l'affichage
 *	(prise en compte du decalage avec la scrollbar)
 *	return get_start_area() + width
 */
int WaveformSystem::get_end_area()
{
	//se_debug(SE_DEBUG_WAVEFORM);

#warning "CHECK verifier que ça marche bien avec la drawing area au lieu de la scrollbar"
	//return get_start_area() + m_hscrollbarWaveform->get_allocation().get_width();
	return get_start_area() + m_drawingWaveform->get_width();
}

/*
 *	retourne la position dans l'area par rapport au temps (du waveform)
 */
int WaveformSystem::get_pos_by_time(gint64 time)
{
	//se_debug(SE_DEBUG_WAVEFORM);

	// passage ne % a partir du waveform
	float percent = ((float)time / (float)m_waveform->get_duration());


	float width = (float)m_drawingWaveform->get_width() * get_zoom();

	// du % on passe a une position dans le widget
	float pos = width * percent;
 
	if(pos < 0) 
		pos = 0;
	else if(pos > width) 
		pos = width;

	return (int)pos;
}

/*
 *	retourne le temps (gstreamer) par rapport a la position (pixel)
 *	sur le waveform
 */
gint64 WaveformSystem::get_time_by_pos(gint64 pos)
{
	//se_debug(SE_DEBUG_WAVEFORM);

	float width = (float)m_drawingWaveform->get_width() * get_zoom();

	float percent = ((float)pos / width);
	
	float time = ((float)m_waveform->get_duration() * percent);
	return (gint64)time;
}


/*
 *
 */
void WaveformSystem::draw_timeline(Cairo::RefPtr<Cairo::Context> &cr, int height)
{
	se_debug(SE_DEBUG_WAVEFORM);

	gint width = m_drawingWaveform->get_width();
	
	// background
	cr->set_source_rgb(1,1,1);
	cr->rectangle(0, 0, width, height);
	cr->fill();
	
	// line
	cr->set_source_rgb(0,0,0);

	cr->move_to(0, height);
	cr->line_to(width, height);
	cr->stroke();

	// time
	gint64 sec_1 = GST_SECOND;
	gint64 sec_5 = GST_SECOND * 5;
	gint64 sec_10 = GST_SECOND * 10;
	
	guint height_1	= height-3;
	guint height_5	= height-5;
	guint height_10	= height-8;

	while( get_pos_by_time(sec_5) < 60)
	{
		sec_1		*=2;
		sec_5		*=2;
		sec_10	*=2;
	}

	draw_timeline_sec(cr, sec_1, height, height_1);
	draw_timeline_sec(cr, sec_5, height, height_5);
	draw_timeline_sec(cr, sec_10, height, height_10);

	draw_timeline_time(cr, sec_5, height);
}

/*
 *
 */
void WaveformSystem::draw_timeline_sec(Cairo::RefPtr<Cairo::Context> &cr, gint64 sec, int y, int upper)
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(sec > 0)
	{
		//cr->set_antialias(Cairo::ANTIALIAS_NONE);

		int startx = get_start_area();
		int endx = get_end_area();

		gint64 start = get_time_by_pos(startx);
		gint64 end = get_time_by_pos(endx);

		gint64 diff = start % sec;

		start -= diff;

		if(get_pos_by_time(start) < startx)
			start += sec;

		for(gint64 t=start; t<end; t+=sec)
		{
			int px = (int)get_pos_by_time(t) - startx;

			cr->move_to(px, y);
			cr->line_to(px, upper);
		}

		cr->stroke();
	}
}

/*
 *
 */
void WaveformSystem::draw_timeline_time(Cairo::RefPtr<Cairo::Context> &cr, gint64 sec, int height)
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(sec > 0)
	{
		cr->set_source_rgb(0,0,0);
		cr->set_font_size(13);

		int startx = get_start_area();
		int endx = get_end_area();

		gint64 start = get_time_by_pos(startx);
		gint64 end = get_time_by_pos(endx);

		gint64 diff = start % sec;

		start -= diff;

		// pour centrer le texte
		//startx -= 25;

		height -= 15;

		Cairo::TextExtents ex;
		cr->get_text_extents(Gst::time_to_string(0), ex);

		for(gint64 t=start; t<end; t+=sec)
		{
			int px = (int)get_pos_by_time(t) - startx;

			cr->move_to(px - ex.width * 0.5, height);
			
			Glib::ustring str = Gst::time_to_string(t);

			cr->show_text(str);
			//cr->text_path(str);
		}

		//cr->fill();
	}
}


/*
 *
 */
void WaveformSystem::draw_channels(Cairo::RefPtr<Cairo::Context> &cr, unsigned int channel, int y, int height)
{
	se_debug(SE_DEBUG_WAVEFORM);

	cr->set_source_rgba(m_color_wave[0], m_color_wave[1], m_color_wave[2], m_color_wave[3]);
	
	guint count = 4;
	
	double peak, scale_peak;
	float scale = m_scale * height;

	int y_height = y + height;
/*
	cr->move_to(0, y + height);

	gint t=0;

	for(t=startx; t<endx; t+=count)
	{
		peak = m_waveform->get_channel(channel, area_to_waveform_channel(t));

		scale_peak = peak * scale;

		if(scale_peak < 0) scale_peak = - scale_peak;
		if(scale_peak > height) scale_peak = height;

		cr->line_to(t - startx, y_height - scale_peak);
	}
	cr->line_to(endx, y+height);
*/
	// optimisation
	{
		// place le point de départ
		cr->move_to(0, y_height);
		
		// recherche le point de départ par rapport au waveform
		double begin = area_to_waveform_channel_2(get_start_area());

		// recherche le prochain point dans le waveform
		// par rapport au décalage (4 pixel)
		// ce qui permet une constante qu'importe le zoom
		double move = area_to_waveform_channel_2(count);

		// la longueur de la surface afin de ne rendre seulement
		// ce qui est visible
		gint lenght = m_drawingWaveform->get_width();
		
		double x=begin;

		for(gint t=0; t<lenght; t+=count, x+=move)
		{
			peak = m_waveform->get_channel(channel, (int)x);
	
			scale_peak = peak * scale;

			if(scale_peak < 0) scale_peak = - scale_peak;
			if(scale_peak > height) scale_peak = height;

			cr->line_to(t, y_height - scale_peak);
		}

		cr->line_to(lenght, y+height);
	}
	// 



	if(m_display_waveform_fill)
	{
		cr->fill_preserve();
		cr->set_source_rgba(m_color_wave_fill[0], m_color_wave_fill[1], m_color_wave_fill[2], m_color_wave_fill[3]);
		cr->stroke_preserve();
	}
	else
		cr->fill();
}

/*
 *
 */
void WaveformSystem::draw_marker(Cairo::RefPtr<Cairo::Context> &cr, const Gdk::Rectangle &rec, gint64 start, gint64 end, float color[4])
{
//	se_debug(SE_DEBUG_WAVEFORM);

	int y = rec.get_y();
	int width = rec.get_width();
	int height = rec.get_height();

	gint64 x = get_pos_by_time(start) - get_start_area();
	gint64 w = get_pos_by_time(end) - get_start_area();

	if(((x >= 0 && x <= width) || (w >=0 && w <= width) || (x <=0 && w >= width)) == false)
		return;

	if(start == end)
		return;

	se_debug(SE_DEBUG_WAVEFORM);

	cr->set_source_rgba(color[0], color[1], color[2], color[3]);
	cr->move_to(0,0);

	if(x > w)
		cr->set_source_rgba(1.0, 0.0, 0.0, 0.5);

	cr->rectangle(x, y, w-x, height);
	cr->fill();
/*
	cr->set_source_rgba(0.3, 0.3, 0.3, 0.5);
	
	cr->move_to(x, y);
	cr->line_to(x, height);

	cr->move_to(w, y);
	cr->line_to(w, height);

	cr->stroke();
*/
}

/*
 *
 */
void WaveformSystem::draw_markers(Cairo::RefPtr<Cairo::Context> &cr, const Gdk::Rectangle &rec, Document *doc)
{
	se_debug(SE_DEBUG_WAVEFORM);

	g_return_if_fail(doc);

	// on recupere le temps visible pour ne dessiner que les markers visibles
	gint64 t_start = get_time_by_pos(get_start_area()) / GST_MSECOND;

	// recuperation des iter qui sont compris dans ce temps
	Gtk::TreeIter iter_start = doc->get_subtitle_model()->find_in_or_after(getTime2MSecs(t_start));
	Gtk::TreeIter iter = doc->get_subtitle_model()->getFirst();
	Gtk::TreeIter selected = doc->get_subtitle_view()->getSelected();

	if(selected)
	{
		for(; iter; ++iter)
		{
			SubtitleModifier sub(iter);

			gint64 s = sub.get_start().totalmsecs * GST_MSECOND;
			gint64 e = sub.get_end().totalmsecs * GST_MSECOND;

			if(iter == selected)
				draw_marker(cr, rec, s,e, m_color_marker_hightlight);
			else
				draw_marker(cr, rec, s,e, m_color_marker);
		}
	}
	else
	{
		for(; iter; ++iter)
		{
			SubtitleModifier sub(iter);

			gint64 s = sub.get_start().totalmsecs * GST_MSECOND;
			gint64 e = sub.get_end().totalmsecs * GST_MSECOND;

			draw_marker(cr, rec, s,e, m_color_marker);
		}
	}
}

/*
 *
 */
void WaveformSystem::init_with_document(Document *doc)
{
	se_debug(SE_DEBUG_WAVEFORM);

	m_currentDocument = NULL;

	// disconnect previous document
	std::map<std::string, sigc::connection>::iterator it;
	
	for(it = m_connection.begin(); it!=m_connection.end(); ++it)
		it->second.disconnect();

	m_connection.clear();
	

	if(doc != NULL)
	{
		m_currentDocument = doc;

#define CONNECT(signal, callback) m_connection[signal] = doc->get_signal(signal).connect( \
			sigc::mem_fun(*this, &WaveformSystem::callback));

		CONNECT("subtitle-selection-changed", on_subtitle_selection_changed);
		CONNECT("subtitle-time-changed", on_subtitle_time_changed);

#undef CONNECT

		init_scrollbarWaveform();
	}

	redraw_marker();
}

/*
 *
 */
void WaveformSystem::on_subtitle_selection_changed()
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(!m_waveform)
		return;

	if(m_scrolling_with_selection && m_playerVideo && m_currentDocument)
	{
		if(!m_playerVideo->is_playing())
		{
			center_with_selected_subtitle();
		}
		else
			redraw_marker();
	}
	else
		redraw_marker();
}

/*
 *
 */
void WaveformSystem::on_subtitle_time_changed()
{
	se_debug(SE_DEBUG_WAVEFORM);

	redraw_marker();
}

/*
 *
 */
void WaveformSystem::on_video_timeout()
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(m_waveform)
	{
		update_view_from_cursor();

		m_drawingWaveform->queue_draw();
	}
}

void WaveformSystem::scroll_to_position(gint position)
{
	se_debug(SE_DEBUG_WAVEFORM);
/*
	if(position < get_start_area())
		m_hscrollbarWaveform->set_value(position);
	else if(position > get_end_area())
		m_hscrollbarWaveform->set_value(position);
*/
	if((position - 20) < get_start_area())
		m_hscrollbarWaveform->set_value(position - 20);
	else if((position + 20) > get_end_area())
		m_hscrollbarWaveform->set_value(position + 20);
}

/*
 *
 */
void WaveformSystem::scroll_to_position_and_center(gint position)
{
	se_debug_message(SE_DEBUG_WAVEFORM, "position=%d", position);

	int start_area = get_start_area();
	int end_area = get_end_area();

	double center_area = start_area + (end_area - start_area) * 0.5;

	double diff = position - center_area;

	m_hscrollbarWaveform->set_value(start_area + diff);
}

/*
 *	place la position de la scrollbar par rapport 
 *	a la lecture
 */
void WaveformSystem::update_view_from_cursor()
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(m_playerVideo == NULL || m_scrolling_with_cursor == false)
		return;

	gint position = get_pos_by_time(m_playerVideo->get_position() * GST_MSECOND);

	scroll_to_position(position);
}


/*
 *
 */
void WaveformSystem::on_action_emit(const Glib::ustring &name)
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(name == "waveform/open")
		on_open_waveform();

	if(!m_waveform)
		return;

	if(name == "waveform/save")
		on_save_waveform();
	else if(name == "waveform/zoom-in")
		zoom_in();
	else if(name == "waveform/zoom-out")
		zoom_out();
	else if(name == "waveform/zoom-selection")
		zoom_selection();
	else if(name == "waveform/zoom-all")
		zoom_all();
	else if(name == "waveform/center-with-selected-subtitle")
		center_with_selected_subtitle();
}

/*
 *
 */
bool WaveformSystem::open(const Glib::ustring &uri)
{
	if(open_media(uri))
	{
		if(m_playerVideo)
		{
			// teste si la video est presente
			//if(Glib::file_test(Glib::filename_from_uri(m_waveform->get_video_uri()), Glib::FILE_TEST_EXISTS))
			{
				m_playerVideo->open(m_waveform->get_video_uri());
				return true;
			}
			//else
			{
#warning "FIXME: find the video for waveform"
			}
				//
		}
	}
	else
	{
		// generate waveform
		WaveformGeneratorUI ui(m_waveform, uri);
			
		if(m_waveform)
		{
			show();
			set_sensitive(true);
			m_hscrollbarWaveform->set_value(0);
			init_scrollbarWaveform();
			m_playerVideo->open(uri);
			return true;
		}
	}
	return false;
}
/*
 *
 */
void WaveformSystem::on_open_waveform()
{
	se_debug(SE_DEBUG_WAVEFORM);

	OpenWaveformUI ui;
	if(ui.run() == Gtk::RESPONSE_OK)
	{
		ui.hide();
		Glib::ustring uri = ui.get_uri();

		open(uri);

	}
}

void WaveformSystem::on_save_waveform()
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(m_waveform)
	{
		Gtk::FileChooserDialog ui(_("Save Waveform"), Gtk::FILE_CHOOSER_ACTION_SAVE);
		ui.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		ui.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
		ui.set_default_response(Gtk::RESPONSE_OK);

		if(ui.run() == Gtk::RESPONSE_OK)
		{
			Glib::ustring uri = ui.get_uri();

			m_waveform->save(uri);
		}
	}
}

void WaveformSystem::on_config_waveform_changed(const Glib::ustring &key, const Glib::ustring &value)
{
	se_debug(SE_DEBUG_WAVEFORM);

#define READ_COLOR(buf) sscanf(value.c_str(), "%f;%f;%f;%f", &buf[0], &buf[1], &buf[2], &buf[3])

	if(key == "color-background")
		READ_COLOR(m_color_background);
	else if(key == "color-play-line")
		READ_COLOR(m_color_play_line);
	else if(key == "color-wave")
		READ_COLOR(m_color_wave);
	else if(key == "color-wave-fill")
		READ_COLOR(m_color_wave_fill);
	else if(key == "color-marker")
		READ_COLOR(m_color_marker);
	else if(key == "color-marker-hightlight")
		READ_COLOR(m_color_marker_hightlight);
	else if(key == "display-waveform-fill")
	{
		bool state;
		from_string(value, state);
		m_display_waveform_fill=state;
	}
	else if(key == "display-background")
	{
		bool state;
		from_string(value, state);
		m_display_background=state;
	}
	else if(key == "scrolling-with-cursor")
	{
		bool state;
		from_string(value, state);
		m_scrolling_with_cursor=state;
	}
	else if(key == "scrolling-with-selection")
	{
		bool state;
		from_string(value, state);
		m_scrolling_with_selection=state;
	}

	redraw_all();

#undef READ_COLOR
}

/*
 *
 */
void WaveformSystem::zoom_in()
{
	set_zoom(m_zoom + 1);
}

/*
 *
 */
void WaveformSystem::zoom_out()
{
	set_zoom(m_zoom - 1);
}

/*
 *
 */
void WaveformSystem::zoom_selection()
{
	if(Document *doc = DocumentSystem::getInstance().getCurrentDocument())
	{
		SubtitleModifier subtitle = doc->get_subtitle_view()->getSelected();
		if(subtitle)
		{
			zoom_in();

			int start = __get_pos_by_time(subtitle.get_start().totalmsecs);
			int end   = __get_pos_by_time(subtitle.get_end().totalmsecs);

			// centre du sous-titre
			int subtitle_center = start + int((end - start) * 0.5);

			scroll_to_position_and_center(subtitle_center);
		}
	}
}

/*
 *
 */
void WaveformSystem::zoom_all()
{
	set_zoom(1);
}

/*
 *	centre l'area sur le sous-titre selectionner
 */
void WaveformSystem::center_with_selected_subtitle()
{
	se_debug(SE_DEBUG_WAVEFORM);

	if(!m_currentDocument)
		return;

	SubtitleModifier subtitle(m_currentDocument->get_subtitle_view()->getSelected());
	if(!subtitle)
		return;

	int start	= __get_pos_by_time(subtitle.get_start().totalmsecs);
	int end		= __get_pos_by_time(subtitle.get_end().totalmsecs);

	int center = start + int((end - start) * 0.5);

	scroll_to_position_and_center(center);
	
	redraw_marker();
}

/*
 *
 */
void WaveformSystem::on_slider_zoom_changed()
{
	double value = m_sliderZoom->get_value();

	set_zoom((int)value);
}

void WaveformSystem::on_slider_scale_changed()
{
	double value = m_sliderScale->get_value();

	set_scale((int)value);
}

/*
 *
 */
bool WaveformSystem::on_slider_scroll_event(GdkEventScroll *ev, Gtk::Scale *scale)
{
	double value = scale->get_value();
	if(ev->direction == GDK_SCROLL_UP)
		scale->set_value(value + 1);
	else if(ev->direction == GDK_SCROLL_DOWN)
		scale->set_value(value - 1);

	return true;
}


